﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

//Buddy-stuff
using Styx;
using Styx.Plugins.PluginClass;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System.Diagnostics;
using System.Xml.Linq;
using System.Net;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;

namespace BuddyProwl
{
    class BuddyProwl : HBPlugin
    {
        public static Config Settings;
        public static bool Init;
        public static Notifo Pusher = new Notifo();

        private Stopwatch pushtimer = new Stopwatch();

        public BuddyProwl()
        {
            Update();
            Settings = new Config();
        }

        public override void Pulse()
        {
            if (!Init)
                Initialise();

            if (Init && !pushtimer.IsRunning)
                pushtimer.Start();

            if (Init && Settings.UseNotifo && pushtimer.Elapsed.Seconds > 3 && Pusher.PushList.Count > 0 && Settings.APIKey.Length == 41 && Settings.Username != "")
            {
                try
                {
                    Notifo.PushMessage Next = Pusher.PushList.Dequeue();
                    Pusher.SendNotification(Next.PushDescription, Next.PushEvent, Next.PushLabel).ToString();

                    Log.toHonorbuddy("White", "Pushed message!");

                    pushtimer.Reset();
                }
                catch (Exception exception)
                {
                    Log.toHonorbuddy("White", "Couldn't push message - " + exception.ToString());
                }
            }
        }

        #region Initialize
        private void Initialise()
        {
            BuddyProwl.Init = true;

            // Load settings
            BuddyProwl.Settings.Load();

            // Set Pusher
            Pusher.Reload(Settings.Username, Settings.APIKey);

            // Handle messages
            Lua.Events.AttachEvent("CHAT_MSG_BN_WHISPER", Events.handleBNWhisper);
            WoWChat.Whisper += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatWhisperEventArgs>(Events.handleWhisper);
            WoWChat.Say += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleSay);
            WoWChat.Yell += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleYell);
            WoWChat.Guild += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatGuildEventArgs>(Events.handleGuild);
            WoWChat.Officer += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleOfficer);
            WoWChat.Party += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleParty);
            WoWChat.Battleground += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleBG);
            WoWChat.Emote += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatAuthoredEventArgs>(Events.handleEmote);
            WoWChat.TextEmote += new WoWChat.ChatMessageHandlerEx<WoWChat.ChatAuthoredEventArgs>(Events.handleTextEmote);

            // Handle Player Events
            BotEvents.Player.OnLevelUp += new BotEvents.Player.LevelUpDelegate(Events.handleLevelUp);
            BotEvents.Player.OnPlayerDied += new BotEvents.Player.PlayerDiedDelegate(Events.handleDeath);

            // Other
            Lua.Events.AttachEvent("ACHIEVEMENT_EARNED", Events.handleAchievement);
            Lua.Events.AttachEvent("CHAT_MSG_LOOT", Events.handleLootOpened);

            Log.toHonorbuddy("White", "Initialized!");
        }
        #endregion

        #region Dispose
        public override void Dispose()
        {
            // Handle messages
            Lua.Events.DetachEvent("CHAT_MSG_BN_WHISPER", Events.handleBNWhisper);
            WoWChat.Whisper -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatWhisperEventArgs>(Events.handleWhisper);
            WoWChat.Say -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleSay);
            WoWChat.Yell -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleYell);
            WoWChat.Guild -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatGuildEventArgs>(Events.handleGuild);
            WoWChat.Officer -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleOfficer);
            WoWChat.Party -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleParty);
            WoWChat.Battleground -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatLanguageSpecificEventArgs>(Events.handleBG);
            WoWChat.Emote -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatAuthoredEventArgs>(Events.handleEmote);
            WoWChat.TextEmote -= new WoWChat.ChatMessageHandlerEx<WoWChat.ChatAuthoredEventArgs>(Events.handleTextEmote);

            // Handle Player Events
            BotEvents.Player.OnLevelUp += new BotEvents.Player.LevelUpDelegate(Events.handleLevelUp);
            BotEvents.Player.OnPlayerDied += new BotEvents.Player.PlayerDiedDelegate(Events.handleDeath);

            // Other
            Lua.Events.DetachEvent("ACHIEVEMENT_EARNED", Events.handleAchievement);
            Lua.Events.DetachEvent("CHAT_MSG_LOOT", Events.handleLootOpened);

            Log.toHonorbuddy("White", "Disposed!");

            Init = false;
            base.Dispose();
        }
        #endregion

        private void Update()
        {
            Log.toHonorbuddy("White", "Checking for updates!");
            try
            {
                string BPPath = Logging.ApplicationPath + @"\Plugins\BuddyProwl";
                string BPUrl = "http://mickur.se/buddyprowl";

                CultureInfo provider = new CultureInfo("sv-SE");

                WebClient client = new WebClient();

                XDocument manifestLatest = XDocument.Load(BPUrl + "/manifest.xml");
                XDocument manifestCurrent = XDocument.Load(BPPath + "\\manifest.xml");
                DateTime latestTime = DateTime.Parse(manifestLatest.Element("Manifest").Element("UpdateTime").Value, provider);
                DateTime currentTime = DateTime.Parse(manifestCurrent.Element("Manifest").Element("UpdateTime").Value, provider);

                if (latestTime <= currentTime)
                    return;

                client.DownloadFile(BPUrl + "/manifest.xml", BPPath + "\\manifest.xml");

                Dictionary<string, string> fileList = new Dictionary<string, string>();
                foreach (XElement fileNode in manifestLatest.Element("Manifest").Descendants("File"))
                {
                    string path = fileNode.Element("Path").Value;
                    string hash = fileNode.Element("Hash").Value;
                    fileList.Add(path, hash);
                }
                
                // Remove files not needed
                foreach (string f in Directory.GetFiles(BPPath))
                {
                    string fn = Path.GetFileName(f);
                    if (!fileList.ContainsKey(@"\" + fn) && fn != "manifest.xml")
                    {
                        File.Delete(f);
                        Log.toHonorbuddy("Red", "Deleted " + fn);
                    }
                }

                int count = 0;

                foreach (KeyValuePair<string, string> file in fileList)
                {
                    string HashCode;
                    if (File.Exists(BPPath + @file.Key))
                    {
                        FileStream stmcheck = File.OpenRead(BPPath + @file.Key);
                        byte[] hash = new MD5CryptoServiceProvider().ComputeHash(stmcheck);
                        HashCode = BitConverter.ToString(hash).Replace("-", "");
                        stmcheck.Close();
                    }
                    else
                        HashCode = "";

                    if (HashCode != file.Value)
                    {
                        count++;
                        if (@file.Key.Contains(@"\") && @file.Key.LastIndexOf(@"\") != 0)
                        {
                            string directoryName = @file.Key.Remove(@file.Key.LastIndexOf('\\'));
                            Directory.CreateDirectory(BPPath + directoryName);
                        }

                        string url = BPUrl + @file.Key;
                        client.DownloadFile(url.Replace('\\', '/'), BPPath + @file.Key);
                        if (HashCode == "")
                            Log.toHonorbuddy("White", "Added " + file.Key);
                        else
                            Log.toHonorbuddy("White", "Updated " + file.Key);
                    }
                }
                if (count > 0)
                {
                    Log.toHonorbuddy("White", "Recompile plugins or restart Honorbuddy to let changes take effect");
                }
            }
            catch
            {
                Log.toHonorbuddy("White", "Couldn't check for updates, maybe the server is down.");
            }
        }

        public override string Name { get { return Database.Plugin_Name; } }
        public override string Author { get { return Database.Plugin_Author; } }
        public override Version Version { get { return Database.Plugin_Version; } }
        public override bool WantButton { get { return Database.Plugin_WantButton; } }
        public override string ButtonText { get { return Database.Plugin_ButtonText; } }

        public override void OnButtonPress()
        {
            // Show Configuration Form
            new frmConfig().ShowDialog();
        }
    }
}
