﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Net;
using System.Threading;
using System.IO;

namespace BuddyProwl
{
    public class Notifo
    {
        public Queue<PushMessage> PushList = new Queue<PushMessage>();

        const string BaseUrl = "https://api.notifo.com/v1/";
        const string SendNotificationUrl = BaseUrl + "send_notification";
        private string _authorizationHeaderValue;

        public Notifo()
        {
        }

        public void Reload(string username, string secret)
        {
            _authorizationHeaderValue = "Basic " + Convert.ToBase64String(Encoding.ASCII.GetBytes(username + ":" + secret));
        }

        public string SendNotification(string msg, string title, string label)
        {
            var request = PrepareRequest(SendNotificationUrl);

            var fields = new Dictionary<string, string>()
                {
                    { "msg", msg },
                    { "title", title },
                    { "label", label },
                };

            WriteValues(request, fields);

            return request.GetResponse().ToString();
        }

        HttpWebRequest PrepareRequest(string url)
        {
            var request = (HttpWebRequest)WebRequest.Create(url);
            request.Method = "POST";
            request.Headers.Add("Authorization", _authorizationHeaderValue);
            request.ContentType = "application/x-www-form-urlencoded";
            request.Accept = "application/json, text/json";
            request.Timeout = Timeout.Infinite;
            return request;
        }

        void WriteValues(WebRequest request, IDictionary<string, string> values)
        {
            using (var requestStream = request.GetRequestStream())
            using (var writer = new StreamWriter(requestStream))
            {
                foreach (var v in values)
                {
                    if (string.IsNullOrEmpty(v.Value))
                        continue;

                    writer.Write(v.Key);
                    writer.Write("=");
                    writer.Write(Uri.EscapeDataString(v.Value));
                    writer.Write("&");
                }
            }
        }

        public struct PushMessage
        {
            public string PushEvent, PushDescription, PushLabel;

            public PushMessage(string Event, string Description, string Label)
            {
                this.PushEvent = Event;
                this.PushDescription = Description;
                this.PushLabel = Label;
            }
        }
    }
}